# -*- coding: utf-8 -*-

"""upload_to_s3
Downloads a file from S3.
Author: ksco92
"""

import boto3


##########################################
##########################################
##########################################
##########################################

def download_from_s3(file_name, aws_bucket_name, aws_file_path, aws_region_name='us-east-1', aws_access_key_id=None,
                     aws_secret_access_key=None):
    """Downloads a file from the specified bucket and path."""

    if aws_access_key_id and aws_secret_access_key:

        s3 = boto3.resource(service_name='s3',
                            aws_access_key_id=aws_access_key_id,
                            aws_secret_access_key=aws_secret_access_key,
                            region_name=aws_region_name)

    else:
        s3 = boto3.resource(service_name='s3',
                            region_name=aws_region_name)

    if aws_file_path == '':
        full_path = file_name
    else:
        full_path = aws_file_path + '/' + file_name

    s3.Bucket(aws_bucket_name).download_file(full_path, file_name)



# -*- coding: utf-8 -*-

"""CursorFactory
Object to return a cursor based on the engine name and connection string parameters.
Author: ksco92"""

import pyodbc
import socket

import pg8000
import pymysql


class CursorFactory:

    def __init__(self, engine, host, port, username, password, schema, autocommit=False):
        self.engine = engine
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.schema = schema
        self.autocommit = autocommit

    ##########################################
    ##########################################

    def create_cursor(self):

        """Creates a cursor based on the engine provided to the runner."""

        if self.engine == 'mysql':
            conn = pymysql.connect(host=self.host, port=self.port, user=self.username, passwd=self.password,
                                   db=self.schema, autocommit=self.autocommit, connect_timeout=36000,
                                   local_infile=True,
                                   max_allowed_packet=16 * 1024, charset='utf8')

        elif self.engine in ('redshift', 'postgresql'):

            conn = pg8000.connect(user=self.username, password=self.password, host=self.host, port=self.port,
                                  database=self.schema)
            conn._usock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)

            if self.autocommit:
                conn.autocommit = True

        elif self.engine == 'sqlserver':

            conn = pyodbc.connect(driver="{SQL Server}", server=self.host + ',' + str(self.port),
                                  database=self.schema,
                                  uid=self.username, pwd=self.password, autocommit=self.autocommit)

        else:
            return None

        return conn.cursor()



# -*- coding: utf-8 -*-

"""ResultsFormatter
Object to format results of a query in format of a nested list to a file, format or S3.
Author: ksco92"""

import pandas as pd

from py_vor.tools.upload_to_s3 import upload_to_s3
from py_vor.tools.write_to_csv import write_to_csv


##########################################
##########################################
##########################################
##########################################

class ResultsFormatter:

    def __init__(self, results, return_as='nested_list', include_headers=True, results_file=None, aws_bucket_name=None,
                 aws_file_path='', aws_region_name='us-east-1', aws_access_key_id=None, aws_secret_access_key=None):
        self.results = results
        self.return_as = return_as
        self.include_headers = include_headers
        self.results_file = results_file
        self.aws_bucket_name = aws_bucket_name
        self.aws_file_path = aws_file_path
        self.aws_region_name = aws_region_name
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key

    ##########################################
    ##########################################

    def return_results_as(self):

        """Returns the results of a statement in the specified format."""

        if self.return_as == 'nested_list':
            if self.include_headers:
                return self.results
            else:
                return self.results[1:]

        elif self.return_as == 'pd_dataframe':
            results = pd.DataFrame(self.results[1:], columns=self.results[0])
            return results

        elif self.return_as == 'csv_file':
            if self.include_headers:
                write_to_csv(self.results_file, self.results)
            else:
                write_to_csv(self.results_file, self.results[1:])
            return None

        elif self.return_as == 'dict':
            keys = self.results[0]
            values = self.results[1:]
            output = []

            for row in values:
                output.append(dict(zip(keys, row)))

            output = {'data': output}
            return output

        elif self.return_as == 's3_upload':
            if self.include_headers:
                write_to_csv(self.results_file, self.results)
            else:
                write_to_csv(self.results_file, self.results[1:])

            upload_to_s3(self.results_file, self.aws_bucket_name, self.aws_file_path,
                         aws_region_name=self.aws_region_name, aws_access_key_id=self.aws_access_key_id,
                         aws_secret_access_key=self.aws_secret_access_key)

            return None



# -*- coding: utf-8 -*-

"""upload_to_s3
Uploads a file to S3.
Author: ksco92
"""

import boto3

##########################################
##########################################
##########################################
##########################################


def upload_to_s3(file_name, aws_bucket_name, aws_file_path, aws_region_name='us-east-1', aws_access_key_id=None,
                 aws_secret_access_key=None):

    """Uploads a file to S3 to a specified bucket and path."""

    if aws_access_key_id and aws_secret_access_key:

        s3 = boto3.resource(service_name='s3',
                            aws_access_key_id=aws_access_key_id,
                            aws_secret_access_key=aws_secret_access_key,
                            region_name=aws_region_name)

    else:
        s3 = boto3.resource(service_name='s3',
                            region_name=aws_region_name)

    with open(file_name, 'rb') as f:
        s3.Bucket(aws_bucket_name).put_object(Key=(aws_file_path + file_name), Body=f)
        f.close()



# -*- coding: utf-8 -*-

"""get_secret
Gets secret from AWS Secrets Manager.
# Author: Somebody in AWS (?)
"""

import boto3
from botocore.exceptions import ClientError
import json

##########################################
##########################################
##########################################
##########################################


def get_secret(secret_name, aws_region_name='us-east-1', aws_access_key_id=None, aws_secret_access_key=None):

    """Gets a secret from AWS Secrets manager based on the secret name."""

    endpoint_url = 'https://secretsmanager.' + aws_region_name + '.amazonaws.com'

    session = boto3.session.Session()

    if aws_access_key_id and aws_secret_access_key:
        client = session.client(service_name='secretsmanager',
                                region_name=aws_region_name,
                                endpoint_url=endpoint_url,
                                aws_access_key_id=aws_access_key_id,
                                aws_secret_access_key=aws_secret_access_key
                                )

    else:
        client = session.client(service_name='secretsmanager',
                                region_name=aws_region_name,
                                endpoint_url=endpoint_url
                                )

    try:
        get_secret_value_response = client.get_secret_value(SecretId=secret_name)

    except ClientError as e:
        if e.response['Error']['Code'] == 'ResourceNotFoundException':
            print("The requested secret " + secret_name + " was not found")
        elif e.response['Error']['Code'] == 'InvalidRequestException':
            print("The request was invalid due to:", e)
        elif e.response['Error']['Code'] == 'InvalidParameterException':
            print("The request had invalid params:", e)

    else:
        # Decrypted secret using the associated KMS CMK
        # Depending on whether the secret was a string or binary, one of these fields will be populated
        if 'SecretString' in get_secret_value_response:
            secret = get_secret_value_response['SecretString']
            return json.loads(secret)

        else:
            binary_secret_data = get_secret_value_response['SecretBinary']
            return json.loads(binary_secret_data)



# -*- coding: utf-8 -*-

"""QueryRunner
Object to run queries from SQL files and return them in a specified format.
Author: ksco92
"""

from py_vor.tools.CursorFactory import CursorFactory
from py_vor.tools.ResultsFormatter import ResultsFormatter


##########################################
##########################################
##########################################
##########################################

class QueryRunner:

    def __init__(self, engine, host, port, username, password, schema, sql_file, autocommit=False, returns_rows=False,
                 text_to_replace=None, replace_text_with=None, include_headers=True, return_as='nested_list',
                 results_file=None, aws_access_key_id=None, aws_secret_access_key=None, aws_region_name='us-east-1',
                 aws_bucket_name=None, aws_file_path='', **kwargs):
        self.engine = engine
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.schema = schema
        self.autocommit = autocommit
        self.returns_rows = returns_rows
        self.sql_file = sql_file
        self.text_to_replace = text_to_replace
        self.replace_text_with = replace_text_with
        self.include_headers = include_headers
        self.return_as = return_as
        self.results_file = results_file
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.aws_region_name = aws_region_name
        self.aws_bucket_name = aws_bucket_name
        self.aws_file_path = aws_file_path

        for key, value in kwargs.items():
            setattr(self, key, value)

    ##########################################
    ##########################################

    def execute_all(self):

        """Executes the results of all the queries in the specified file and returns the results of the last one in the
        specified format."""

        cur = self.get_cursor()
        statements = self.get_statements()
        results = []

        for statement in statements:
            results = self.execute_statement(cur, statement)

        return self.return_results_as(results)

    ##########################################
    ##########################################

    def get_cursor(self):

        """Gets a cursor based on the engine name."""

        return CursorFactory(self.engine, self.host, self.port, self.username, self.password, self.schema,
                             autocommit=self.autocommit).create_cursor()

    ##########################################
    ##########################################

    def execute_statement(self, cur, statement):

        """Executes a SQL statement using the provided cursor."""

        query_results = []
        column_names = []

        cur.execute(statement)

        if self.returns_rows:

            for col in cur.description:
                if self.engine in ('mysql', 'sqlserver'):
                    column_names.append(col[0])
                elif self.engine in ('redshift', 'postgresql'):
                    column_names.append(col[0].decode("utf-8"))

            query_results.append(column_names)

            for row in cur:
                row = [str(n) for n in row]
                query_results.append(list(row))

        else:
            return None

        return query_results

    ##########################################
    ##########################################

    def get_statements(self):

        """Reads the queries in the provided file and replaces the desired text."""

        with open(self.sql_file) as f:
            query = f.read()
            f.close()

        if self.text_to_replace and self.replace_text_with:
            for ttr, rtw in zip(self.text_to_replace, self.replace_text_with):
                query = query.replace(ttr, rtw)

        queries = query.split(';')
        queries = filter(None, queries)

        return queries

    ##########################################
    ##########################################

    def return_results_as(self, results):

        """Returns the results of a statement in the specified format."""

        return ResultsFormatter(results, return_as=self.return_as, include_headers=self.include_headers,
                                results_file=self.results_file, aws_bucket_name=self.aws_bucket_name,
                                aws_file_path=self.aws_file_path, aws_region_name=self.aws_region_name,
                                aws_access_key_id=self.aws_access_key_id,
                                aws_secret_access_key=self.aws_secret_access_key).return_results_as()



# -*- coding: utf-8 -*-

"""Main setup file"""

from setuptools import setup, find_packages

##########################################
##########################################
##########################################
##########################################

setup(name='py_vor',
      version='0.0.6',
      description='BI automation utilities.',
      url='https://github.com/ksco92/py_vor',
      author='Rodrigo Carvajal',
      author_email='rodrigocf_92@hotmail.com',
      license='MIT',
      packages=find_packages(),
      zip_safe=False,
      install_requires=['boto3',
                        'pandas',
                        'pyodbc',
                        'pymysql',
                        'pg8000'
                        ]
      )
